"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const toSet_1 = require("./toSet");
function except(left, right) {
    internal_1.assert.mustBeQueryable(left, "left");
    internal_1.assert.mustBeQueryable(right, "right");
    return internal_1.FlowHierarchy(new ExceptIterable(internal_1.ToIterable(left), internal_1.ToIterable(right)), left);
}
exports.except = except;
let ExceptIterable = class ExceptIterable {
    constructor(left, right) {
        this._left = left;
        this._right = right;
    }
    *[Symbol.iterator]() {
        const set = toSet_1.toSet(this._right);
        for (const element of this._left) {
            if (internal_1.TryAdd(set, element)) {
                yield element;
            }
        }
    }
};
ExceptIterable = tslib_1.__decorate([
    internal_1.ToStringTag("ExceptIterable")
], ExceptIterable);
