"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const internal_1 = require("../internal");
const toSetAsync_1 = require("./toSetAsync");
function intersectAsync(left, right) {
    internal_1.assert.mustBeAsyncQueryable(left, "left");
    internal_1.assert.mustBeAsyncQueryable(right, "right");
    return internal_1.FlowHierarchy(new AsyncIntersectIterable(internal_1.ToPossiblyAsyncIterable(left), internal_1.ToPossiblyAsyncIterable(right)), left, right);
}
exports.intersectAsync = intersectAsync;
let AsyncIntersectIterable = class AsyncIntersectIterable {
    constructor(left, right) {
        this._left = left;
        this._right = right;
    }
    [Symbol.asyncIterator]() {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var e_1, _a;
            const set = yield tslib_1.__await(toSetAsync_1.toSetAsync(this._right));
            if (set.size <= 0) {
                return yield tslib_1.__await(void 0);
            }
            try {
                for (var _b = tslib_1.__asyncValues(this._left), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                    const element = _c.value;
                    if (set.delete(element)) {
                        yield yield tslib_1.__await(element);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
};
AsyncIntersectIterable = tslib_1.__decorate([
    internal_1.ToStringTag("AsyncIntersectIterable")
], AsyncIntersectIterable);
