"use strict";
/*!
  Copyright 2018 Ron Buckton (rbuckton@chronicles.org)

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
/** @module "iterable-query/fn" */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var internal_1 = require("../internal");
var collections_1 = require("../collections");
var toSetAsync_1 = require("./toSetAsync");
function symmetricDifferenceAsync(left, right) {
    internal_1.assert.mustBeAsyncQueryable(left, "left");
    internal_1.assert.mustBeAsyncQueryable(right, "right");
    return internal_1.FlowHierarchy(new AsyncSymmetricDifferenceIterable(internal_1.ToPossiblyAsyncIterable(left), internal_1.ToPossiblyAsyncIterable(right)), left, right);
}
exports.symmetricDifferenceAsync = symmetricDifferenceAsync;
var AsyncSymmetricDifferenceIterable = /** @class */ (function () {
    function AsyncSymmetricDifferenceIterable(left, right) {
        this._left = left;
        this._right = right;
    }
    AsyncSymmetricDifferenceIterable.prototype[Symbol.asyncIterator] = function () {
        return tslib_1.__asyncGenerator(this, arguments, function _a() {
            var e_1, _a, e_2, _b, right, set, _c, _d, element, e_2_1, right_1, right_1_1, element, e_1_1;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0: return [4 /*yield*/, tslib_1.__await(toSetAsync_1.toSetAsync(this._right))];
                    case 1:
                        right = _e.sent();
                        set = new collections_1.Set();
                        _e.label = 2;
                    case 2:
                        _e.trys.push([2, 9, 10, 15]);
                        _c = tslib_1.__asyncValues(this._left);
                        _e.label = 3;
                    case 3: return [4 /*yield*/, tslib_1.__await(_c.next())];
                    case 4:
                        if (!(_d = _e.sent(), !_d.done)) return [3 /*break*/, 8];
                        element = _d.value;
                        if (!(internal_1.TryAdd(set, element) && !right.has(element))) return [3 /*break*/, 7];
                        return [4 /*yield*/, tslib_1.__await(element)];
                    case 5: return [4 /*yield*/, _e.sent()];
                    case 6:
                        _e.sent();
                        _e.label = 7;
                    case 7: return [3 /*break*/, 3];
                    case 8: return [3 /*break*/, 15];
                    case 9:
                        e_2_1 = _e.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 15];
                    case 10:
                        _e.trys.push([10, , 13, 14]);
                        if (!(_d && !_d.done && (_b = _c.return))) return [3 /*break*/, 12];
                        return [4 /*yield*/, tslib_1.__await(_b.call(_c))];
                    case 11:
                        _e.sent();
                        _e.label = 12;
                    case 12: return [3 /*break*/, 14];
                    case 13:
                        if (e_2) throw e_2.error;
                        return [7 /*endfinally*/];
                    case 14: return [7 /*endfinally*/];
                    case 15:
                        _e.trys.push([15, 21, 22, 23]);
                        right_1 = tslib_1.__values(right), right_1_1 = right_1.next();
                        _e.label = 16;
                    case 16:
                        if (!!right_1_1.done) return [3 /*break*/, 20];
                        element = right_1_1.value;
                        if (!internal_1.TryAdd(set, element)) return [3 /*break*/, 19];
                        return [4 /*yield*/, tslib_1.__await(element)];
                    case 17: return [4 /*yield*/, _e.sent()];
                    case 18:
                        _e.sent();
                        _e.label = 19;
                    case 19:
                        right_1_1 = right_1.next();
                        return [3 /*break*/, 16];
                    case 20: return [3 /*break*/, 23];
                    case 21:
                        e_1_1 = _e.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 23];
                    case 22:
                        try {
                            if (right_1_1 && !right_1_1.done && (_a = right_1.return)) _a.call(right_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 23: return [2 /*return*/];
                }
            });
        });
    };
    AsyncSymmetricDifferenceIterable = tslib_1.__decorate([
        internal_1.ToStringTag("AsyncSymmetricDifferenceIterable")
    ], AsyncSymmetricDifferenceIterable);
    return AsyncSymmetricDifferenceIterable;
}());
